<?php

/******************************************************
  php-bibHTML - a bibtex to HTML conversion library.
		built upon the Bibliophile BibTex parsers v2.2,
		http://bibliophile.sourceforge.net/downloads.php

  (c) 2012-2018, Hugo Jonker (hugo.jonker@ou.lu)

  Main usage:
  	<?php
	include_once('php-bibHTML/php-bibHTML.php');
	print PHPBibHTML::bib_to_htmllist("/path/to/bibfile.bib") 
	?>
  This results in HTML output of a list of all REVIEWED bibtex entries
  in bibfile.bib. The list is split per year.

  Other split options:
  - PHPBibHTML::bib_to_htmllist("file.bib", SPLITONTYPETHENYEAR);
  - PHPBibHTML::bib_to_htmllist("file.bib", SPLITONTYPE);
  - PHPBibHTML::bib_to_htmllist("file.bib", SPLITONYEAR);
  - PHPBibHTML::bib_to_htmllist("file.bib", SPLITONYEARTHENTYPE);
  - PHPBibHTML::bib_to_htmllist("file.bib", DONTSPLIT);

  Output is templated, see the various files in the template directory

  Auxiliary (useful) functions:
  - PHPBibHTML::latex_to_html_string(str)
  	converts a latex string to HTML (mostly replacing accented chars)
  - PHPBibHTML::fields_missing(file)
  	returns array with required fields missing from the input file
  - PHPBibHTML::missing_fields(array)
  	returns which required fields are missing from the input bibtex entry


  Dependencies:
  - conf.php 		 - configuration
  - biblink.php		 - to show sanitised bibtex entries
  - TEMPLATEDIR          - holds all templates
  - inc-bibparse.php
    inc-parseauthors.php
    			 - BibTex parser by Grimshaw et al.

  License/usage:
  This program is released under the terms of the GPL v3 license,
  a copy of which should be included with this distribution.

  This program is provided "AS IS", without any warranties of any kind,
  either expressed or implied.

******************************************************/

include_once('conf.php');		// configuration
include_once('inc-parsebib.php');	// bibtex file parser
include_once('inc-parseauthors.php');	// bibtex author format parser


// Limit scope of function names (namespaces only work from php 5.4 on)
class PHPBibHTML {

// fake constructor to avoid errors
function PHPBibHTML() {
	$bibsourcefile = '';
}

/** get_latex_initial
 * Get the initial of a latex string
 *
 * input: latex string
 * output: first letter of the LaTex string
 *
 * Examples (3 cases):
 * 1. X; 2. \"e; 3. \~{a} 
 */
function get_initial($str)  {
	$init = substr($str, 0, 1);

	// case 2 or 3?
	if ( $init == '\\' ) {
		// add next two chars
		$init .= substr($str, 1, 2);

		// case 3?
		if (substr($init, -1) == '{') {
			$init .= substr($str, 3,2);
		}
	}

	return $init;
}

/** canonalise_authors
 * Fix a bibtex author string for readability 
 *
 * input: a bibtex author string
 * output: an HTML author string.
 *
 * Example:
 * author = { Hugo L. Jonker and J. Pang & Naipeng von Dong}
 * output: H.L. Jonker, J. Pang and N. von Dong
 */
function canonalise_authors($str)  {
	$authparse = new PARSECREATORS;
	$autharray = $authparse->parse($str);

	$ret = '';

	foreach($autharray as $auth)  {
		 $inits = '';
		if( !empty($auth['firstname']) ) {
			$inits = PHPBibHTML::get_initial($auth['firstname']) . '.';
		}
		if( !empty($auth['initials']) )  {
			$inits .= str_replace(' ', '.', $auth['initials']);
			$inits .= '.';
		}
		$ret .= $inits . ' ' . 
			(empty($auth['prefix']) ? '' : $auth['prefix'] .  ' ') .
			$auth['surname'] . ' and ';
	}

	// chop off last " and "
	$ret = substr($ret, 0, -5);

	// Convert all-but-last " and " to commas
	$numfields = sizeof(explode(' and ', $ret)) - 1;
	if ($numfields > 1) {
		$arrstr = explode(' and ', $ret, $numfields);
		$ret = implode(', ', $arrstr);
	}

	return PHPBibHTML::latex_to_html_string($ret);
}

/** Latex_to_html_string
 *  converts a latex string to an HTML string using $mapfile
 * 
 *  input: a latex string
 *  output: an HTML string with accents and characters replaced
 * 
 *  example: "\'e" --> "&eacute;"
 * 
 *  imports tex to HTML mapping from $mapfile, format:
 * 	LINE ::= HTML_OPCODE TAB HTML_UNICODE TAB TEX_OPCODE
 * 
 *  ### NOTE: This is *NOT* perfect!
 *            This function does not do tokenization, and is fragile wrt.
 *            newcommands (e.g., \o will _always_ be substituted, since
 *            there exists at least one entry starting with \o in
 *            $mapfile.)
 * 
 *            Note: longer matches in $mapfile take priority -- a property
 *            of strtr().
 */
function latex_to_html_string($str)  {
	global $mapfile;
	static $texhtml = array();

	// only load file if needed
	if (count($texhtml) == 0)  {
		if (strpos($str, '\url{') !== FALSE)  {
			$str = preg_replace('/\\\\url{([^}]*)}/', '\1', $str);
		}

		// chop off last newline, replace others with tabs for explode
		$bigstr = file_get_contents($mapfile);
		$bigstr = substr(str_replace("\n", "\t", $bigstr), 0,
			strlen($bigstr) - 1);
		$bigarr = explode("\t", $bigstr);
		$bigarrsz = count($bigarr);

		// Array is now [i]: HTML; [i+1]: unicode; [i+2]: latex
		// Array is now [i]: latex; [i+1]: HTML; [i+2]: unicode
		// Parse that into sth convenient for strtr
		for($i = 0; $i < $bigarrsz; $i = $i + 3)  {
			$texhtml[$bigarr[$i]] = $bigarr[$i+1];
		}

		// remove latex grouping symbols
		$texhtml['{'] = '';
		$texhtml['}'] = '';
	}


	// Now return the strtr -- strtr favours larger matches, thus
	// allowing us to translate both \o and \omicron in one go
	return strtr($str, $texhtml);
}

/** l2h
 *  Identical to latex_to_html_string, but shorter to type :)
 */
function l2h($str)  {
	return PHPBibHTML::latex_to_html_string($str);
}

/** fields_missing:	FILE
 *  returns required fields missing in the bibtex file FILE
 * 
 *  input: a bibtex filename
 *  output: 
 *   - an array ($bibtexkey => array(missing fields)),
 *   - FALSE if no missing entries,
 *   - -1 if file could not be read.
 */
function fields_missing($file)  {
	$misfields = array();
	$ret = array();

	if (!is_readable($file))  {
		trigger_error("php-bibHTML: couldn't read bibtexfile.\n");
		$ret = -1;
	} else {
		// parse the file
		$errors = ini_get('error_reporting');
		error_reporting($errors ^ E_NOTICE);
		$parse = new PARSEENTRIES();
		$parse->expandMacro = TRUE;
		$parse->openBib($file);
		$parse->extractEntries();
		$parse->closeBib();
		list($preamble, $strings, $entries, $undefinedStrings) =
			$parse->returnArrays();
		error_reporting($errors);


		// check each of the entries for errors
		foreach($entries as $entry)  {
			$misfields = PHPBibHTML::missing_fields($entry);
			if (!empty($misfields))  {
				$ret[$entry['bibtexCitation']] = $misfields;
			}
		}

	}

	return (empty($ret) ? FALSE : $ret);
}

/** missing_fields:	ARRAY
 *  returns (an array of) required fields missing from the input bibtex array ARRAY
 * 
 *  input: an array of one single parsed bibtex entry, mapping fields to values
 *  	i.e. array( 'bibtexEntryType' => article, 'author' => 'me',  ...)
 *  output: an array of the missing fields, if any, or FALSE if none.
 */
function missing_fields($bibtex)  {
	$reqfields = array(
		'article' => array('author', 'title', 'journal', 'year'),
		'book' => array('author', 'title', 'publisher', 'year'),
		'booklet' => array('title'),
		'conference' => array('author', 'title', 'booktitle', 'year'),
		'inbook' => array('author', 'title', 'chapter', 'pages',
			'publisher', 'year'),
		'incollection' => array('author', 'title', 'booktitle',
			'year'),
		'inproceedings' => array('author', 'title', 'booktitle',
			'year'),
		'manual' => array('title'),
		'mastersthesis' => array('author', 'title', 'school', 'year'),
		'misc' => array(),
		'phdthesis' => array('author', 'title', 'school', 'year'),
		'proceedings' => array('title', 'year'),
		'techreport' => array('author', 'title', 'institution',
			'year'),
		'unpublished' => array('author', 'title', 'note')
	);

	if (!array_key_exists('bibtexEntryType', $bibtex) ||
		!array_key_exists($bibtex['bibtexEntryType'], $reqfields))  {
		trigger_error('php-bibHTML: missingfields(): cannot discern ' .
			'the bibtex-type of the argument; the argument\'s ' .
			'field "bibtexEntryType" missing or not recognised' .
			".\n");
	} else {
		$type = $bibtex['bibtexEntryType'];
		foreach($reqfields[$type] as $field)  {
			if(!array_key_exists($field, $bibtex))  {
				if ($field == 'author' &&
					($type=='book' || $type == 'inbook')
				   && array_key_exists('editor', $bibtex)
				)  {
					continue;
				}
				if ($type == 'inbook' && $field == 'chapter' 
				   && array_key_exists('pages', $bibtex))  {
					continue;
				}
				if ($type == 'inbook' && $field == 'pages' 
				   && array_key_exists('chapter', $bibtex))  {
					continue;
				}
				$ret[] = $field;
			}
		}
	}

	return (empty($ret) ? FALSE : $ret);
}
				
				
/** bib2html
 *  converts one (parsed) bibtex entry into an HTML string
 * 
 *  input: an array of one single parsed bibtex entry, mapping fields to values
 *  	i.e. array( 'author' => 'me', 'year' => 1900, ...)
 *  output: an HTML-formatted string for this entry
 */
function bib2html($bibtex)  {
	$ret = '';

	// Check for missing fields
	$misfields = PHPBibHTML::missing_fields($bibtex);
	if (!empty($misfields))  {
		trigger_error('php-bibHTML: missing required field(s) in ' .
			'entry ' . $bibtex['bibtexCitation'] .
			'. Missing fields: ' .
			implode(', ', $misfields) . ".\n");
	}

	// Set default values for template vars
	$address = (array_key_exists('address', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['address']) : '' );
	$annote = (array_key_exists('annote', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['annote']) : '' );
	$author = (array_key_exists('author', $bibtex) ? 
			PHPBibHTML::canonalise_authors($bibtex['author']) : '');
	$bktitle = (array_key_exists('booktitle', $bibtex) ?
			PHPBibHTML::l2h($bibtex['booktitle']) : '' );
	$chap = (array_key_exists('chapter', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['chapter']) : '' );
	$edition = (array_key_exists('edition', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['edition']) : '' );
	$editor = (array_key_exists('editor', $bibtex) ? 
			PHPBibHTML::canonalise_authors($bibtex['editor']) : '');
	$howpub = (array_key_exists('howpublished', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['howpublished']) : '' );
	$inst = (array_key_exists('institution', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['institution']) : '' );
	$journal = (array_key_exists('journal', $bibtex) ?
			PHPBibHTML::l2h($bibtex['journal']) : '');
	$key = (array_key_exists('key', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['key']) : '' );
	$month = (array_key_exists('month', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['month']) : '' );
	$note = (array_key_exists('note', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['note']) : '' );
	$num = (array_key_exists('number', $bibtex) ?
			$bibtex['number'] : '');
	$org = (array_key_exists('organization', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['organization']) : '' );
	$pg = (array_key_exists('pages', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['pages']) : '' );
	$pub = (array_key_exists('publisher', $bibtex) ?
			PHPBibHTML::l2h($bibtex['publisher']) : '');
	$school = (array_key_exists('school', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['school']) : '' );
	$sry = (array_key_exists('series', $bibtex) ?
			PHPBibHTML::l2h($bibtex['series']) : '');
	$title = (array_key_exists('title', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['title']) : '');
	$type = (array_key_exists('type', $bibtex) ? 
			PHPBibHTML::l2h($bibtex['type']) : '' );
	$vol = (array_key_exists('volume', $bibtex) ?
			$bibtex['volume'] : '');
	$year = (array_key_exists('year', $bibtex) ?
			$bibtex['year'] : '');
	
	// Extra 's' in case of multiple editors
	$s = (strpos($editor, 'and') > 0 ? 's' : '');

	// Type of the bibtex entry
	$bibtype = $bibtex['bibtexEntryType'];

	switch($bibtype)  {
	case 'article':
	case 'conference':
	case 'inbook':
	case 'incollection':
	case 'inproceedings':
	case 'phdthesis':
	case 'book':
	case 'proceedings':
	case 'misc':
	case 'manual':
	case 'booklet':
	case 'mastersthesis':
	case 'techreport':
	case 'unpublished':
		include(TEMPLATEDIR . $bibtype . '.tmpl');
		break;
	default:
		trigger_error('php-bibHTML: bib2html(): ' .
			"illegal bibtex entry type $bibtype.\n", E_USER_ERROR);
	}

	// Load template
	$ret = $phpbibHTML_bibentry_tmpl . "\n";

	// Clean up.
	// In some cases (empty bibtex fields, incorrect templates),
	// " ,", ". ," and ", ." may appear in output. output space can be
	// any form of white space, including newlines.
 	$ret = preg_replace("/\.(\s|\R)*,/", '.', $ret);
 	$ret = preg_replace("/,(\s|\R)*\./", '.', $ret);
 	$ret = preg_replace("/(\s|\R)*,/", ',', $ret);

	// add links to pdf / slides / bibtex / etc.
	// slides etc. are detected based on extra fields in the bib source
	$ret .= PHPBibHTML::extra_links_str($bibtex);

	return $ret;
}

/** extra_links_str
 *  returns a string containing "extra links" of a bibtex entry.
 *  names of these links determined by conf var $biblink
 * 
 *  Input:	one bibtex entry
 *  Output:	a string containing at least a link to the plain biblisting.
 *  Example:	[ <a href="biblink.php?bib=BIBFILE&id=ID">bib</a> ]
 * 
 *  Currently adds links for the following bibfields:
 * 	preprint, url, ee, doi, slides
 */
function extra_links_str($bibtex)  {
	global $biblink;	// how to label bibfields ee, url, etc.
	global $bibsourcefile;

	// no links if target is empty
	if ($bibsourcefile == '')  {
		return;
	}

	include(TEMPLATEDIR . "biblinks.tmpl");

	$ret = $biblinkstart;
	$ret .= '<a href="' . BIBLINKFILE . '?bibfile=' . $bibsourcefile .
		'&id=' . $bibtex['bibtexCitation'] . '">bib</a>';

	foreach($biblink as $biblabel => $htmllabel)  {
		if(array_key_exists($biblabel, $bibtex))  {
			if($biblabel == "doi")  {
				$doi = "http://dx.doi.org/";
			} else {
				$doi = '';
			}
			$ret .= $biblinkseparator . '<a href="' . $doi .
				$bibtex[$biblabel] . '">' . $htmllabel .
				'</a>';
		}
	}

	return $ret . $biblinkend;
}

/** reversetypes()
 *  return a mapping from bibtype to label
 *  basically swaps the configuration var $bibtypes around
 */
function reversetypes()  {
	global $bibtypes;

	$ret = array();

	if (!is_array($bibtypes))  {
		trigger_error('php-bibHTML: reversetypes(): ' .
			"$bibtypes not an array.\n", E_USER_ERROR);
		return FALSE;
	}

	foreach($bibtypes as $label => $types)  {
		foreach($types as $type)  {
			if (array_key_exists($type, $ret))  {
				trigger_error('php-bibHTML: config error: ' .
					$type . ' assigned multiple labels ' . 
					'in $bibtypes.' . "\n",
					E_USER_ERROR);
				return FALSE;
			}
			$ret[$type] = $label;
		}
	}

	return $ret;
}

/** parsebibfile(BIBFILE)
 * returns an array of bib entries parsed from BIBFILE
 */
function parsebibfile($bibfile)  {

	// test if bibfile is a file
	if (!is_readable($bibfile) || substr($bibfile, -4) != '.bib')  {
		trigger_error("php-bibHTML: bibfile not a valid, readable " .
			".bib file.\n", E_USER_ERROR);
		return FALSE;
	}

	// *** start parsing the file
	// Stop notices to prevent warnings on screen in case of
	// missing fields in bib
	$errors = ini_get('error_reporting');
	error_reporting($errors ^ E_NOTICE);

	// instantiate bibtex parser bibtex 
	$parse = new PARSEENTRIES();
        $parse->expandMacro = TRUE;

	// Parse bibtex file and load result into arrays
	$parse->openBib($bibfile);
	$parse->extractEntries();
	$parse->closeBib();

	// Restore error reporting
	error_reporting($errors);
	// *** parsing done

	return $parse->returnArrays();
}

function splitentries($entries, $split = SPLITONYEAR)  {
	global $rvwkeys;
	global $bibtypes;

	// generate mapping from bibtex types to fields
	$revtypes = PHPBibHTML::reversetypes();

	// split reviewed entries in reviewd
	$reviewd = array(); $labels = array();

	// Enforce the order from $bibtypes
	if ($split == SPLITONTYPE || $split == SPLITONTYPETHENYEAR)  {
		foreach($bibtypes as $label => $bibtypelist)  {
			$reviewd[$label] = array();
		}
	}

	foreach($entries as $entry)  {
		$tp = $entry['bibtexEntryType'];
		$label = $revtypes[$tp];
		$labels[$label] = true;		//remember labels seen
		$yr = (isset($entry['year']) ? $entry['year'] : 'other');
		if (in_array($tp, $rvwkeys))  {
			switch($split) {
			case SPLITONTYPETHENYEAR:
				$reviewd[$label][$yr][] = $entry;
				break;
			case SPLITONYEARTHENTYPE:
				$reviewd[$yr][$label][] = $entry;
				break;
			case SPLITONYEAR:
				$reviewd[$yr][] = $entry;
				break;
			case SPLITONTYPE:
				$reviewd[$label][$yr][] = $entry;
				break;
			default:
				$reviewd[$yr][] = $entry;
			}
		}
	}
	
	return array($reviewd, $labels);
}

function sortentries($entries, $sort=SPLITONYEAR)  {
	global $bibtypes;

	list($reviewd, $labels) = PHPBIBHTML::splitentries($entries, $sort);

	// Get list of all bibtypes to list (all keys appearing in $labels)
	$res = array_intersect_key($bibtypes, $labels);

	switch($sort) {
	case SPLITONTYPETHENYEAR:
	case SPLITONTYPE:	// Merge in printing
		foreach($reviewd as $label => $listbyyear)  {
			if(!empty($listbyyear))  {
				krsort($listbyyear);
				$reviewd[$label] = $listbyyear;
			} else {
				unset($reviewd[$label]);
			}
		}
		break;
	case SPLITONYEARTHENTYPE:
		krsort($reviewd);
		$new = array();
		foreach($reviewd as $year => $labellist)  {
			// enforce order from $bibtypes
			foreach($bibtypes as $label => $bibtypelist)  {
				if (array_key_exists($label, $labellist))  {
					$new[$year][$label] = array();
				}
			}
			// now sort properly
			foreach($labellist as $label => $entrylist)  {
				foreach($entrylist as $entry)  {
					$new[$year][$label][] = $entry;
				}
			}
		}
		$reviewd = $new;
		break;
	case SPLITONYEAR:
	case DONTSPLIT:
	default:
		krsort($reviewd);
	}

	return $reviewd;
}


// returns a list of bibentries split by year (descending)
//	each year is then split further by type
//	order of types is given by $bibtypes in conf file
function genlist_onyear_thentype($reviewd) {
	global $bibtypes;
	$split = SPLITONYEARTHENTYPE;
	$ret = '';
	$cssfile = CSSFILE;

	foreach($reviewd as $year => $labellist)  {
		$yearid = $year;
		$type = '';
		$typeid = '';

		// load & append type template
		include(TEMPLATEDIR . 'list-entries.tmpl');
		$ret .= $phpbibHTML_year_start . "\n";

		foreach($labellist as $type => $entrylist)  {
			$typeid = str_replace(' ', '_', $type) . $yearid;
			include(TEMPLATEDIR . 'list-entries.tmpl');
			$ret .= "\n" . $phpbibHTML_type_start . "\n";

			foreach($entrylist as $entry)  {
				$ret .= $phpbibHTML_entry_start . "\n";
				$ret .= PHPBibHTML::bib2html($entry);
				$ret .= $phpbibHTML_entry_end . "\n";
			}
			$ret .= $phpbibHTML_type_end . "\n";
		}
		$ret .= $phpbibHTML_year_end . "\n";
	}

	return $phpbibHTML_header . "\n" . $ret . "\n" .
		$phpbibHTML_footer . "\n";
}

// returns a string containing a list of the bibentries split by type,
// within each type split by year (descending)
function genlist_ontype_thenyear($reviewd)  {
	global $bibtypes;
	$ret = '';
	$cssfile = CSSFILE;
	$split = SPLITONTYPETHENYEAR;

	foreach($reviewd as $labeltype => $yrlist)  {
		// set vars used in templates
		$type = $labeltype;
		$typeid = str_replace(' ', '_', $type);
		$yearid = '';
		$year = '';

		// load & append type template
		include(TEMPLATEDIR . 'list-entries.tmpl');
		$ret .= $phpbibHTML_type_start . "\n";
			
		foreach($yrlist as $year => $entrylist)  {
			$yearid = $typeid . 'y' . $year;

			// load year template & append
			include(TEMPLATEDIR . 'list-entries.tmpl');
			$ret .= "\n" . $phpbibHTML_year_start . "\n";
			foreach($entrylist as $entry)  {
				$ret .= $phpbibHTML_entry_start . "\n";
				$ret .= PHPBibHTML::bib2html($entry);
				$ret .= $phpbibHTML_entry_end . "\n";
			}
			$ret .= $phpbibHTML_year_end . "\n";

		}
		$ret .= $phpbibHTML_type_end . "\n";
	}

	return $phpbibHTML_header . "\n" . $ret . "\n" .
		$phpbibHTML_footer . "\n";
}

// returns a string containing a list of bibentries split by year
function genlist_onyear($reviewd) {
	global $bibtypes;
	$ret = '';
	$cssfile = CSSFILE;
	$split = SPLITONYEAR;

	// split on year, not on type
	$typeid = ''; $type = '';
	foreach($reviewd as $year => $pubs) {
		$yearid = $year;
		include(TEMPLATEDIR . 'list-entries.tmpl');
		$ret .= "\n" . $phpbibHTML_year_start . "\n";
		foreach($pubs as $entry)  {
			$ret .= $phpbibHTML_entry_start . "\n";
			$ret .= PHPBibHTML::bib2html($entry);
			$ret .= $phpbibHTML_entry_end . "\n";
		}
		$ret .= $phpbibHTML_year_end . "\n";
	}

	return $phpbibHTML_header . "\n" . $ret . "\n" .
		$phpbibHTML_footer . "\n";
}

// returns a string that contains a list of bibentries split by type
function genlist_ontype($reviewd)  {
	// also patch the php-BIBHTML.inc template!
	global $bibtypes;
	$ret = '';
	$cssfile = CSSFILE;
	$split = SPLITONTYPE;
	$year='not split on year';

	// split on type, not by year
	$typeid = ''; $type = '';
	foreach($reviewd as $type => $yr) {
		// set vars used in templates
		$typeid = str_replace(' ', '_', $type);

		include(TEMPLATEDIR . 'list-entries.tmpl');
		$ret .= "\n" . $phpbibHTML_type_start . "\n";
		foreach($yr as $entrylist)  {
			foreach($entrylist as $entry)  {
				$ret .= $phpbibHTML_entry_start . "\n";
				$ret .= PHPBibHTML::bib2html($entry);
				$ret .= $phpbibHTML_entry_end . "\n";
			}
		}
		$ret .= $phpbibHTML_type_end . "\n";
	}

	return $phpbibHTML_header . "\n" . $ret . "\n" .
		$phpbibHTML_footer . "\n";
}

// returns a list of bib entries, without splitting the list
//	the list will be sorted by year (descending)
function genlist_nosplit($reviewd)  {
	// also patch the php-BIBHTML.inc template!
	global $bibtypes;
	$ret = '';
	$cssfile = CSSFILE;
	$split = DONTSPLIT;

	// don't split
	$typeid = ''; $type = '';
	foreach($reviewd as $pubs) {
		include(TEMPLATEDIR . 'list-entries.tmpl');
		foreach($pubs as $entry)  {
			$ret .= $phpbibHTML_entry_start . "\n";
			$ret .= PHPBibHTML::bib2html($entry);
			$ret .= $phpbibHTML_entry_end . "\n";
		}
	}

	return $phpbibHTML_header . "\n" . $ret . "\n" .
		$phpbibHTML_footer . "\n";
}


/** bib_to_htmllist( BIBfile, splitbytype )
 *  returns an HTML string of the entries in BIBfile, grouped by year
 *  	HTML formatting handled in templates from TEMPLATEDIR
 * 
 *  splitbytype: whether to make subgroups by type (articles,
 * 	proceedings, ...) or not.
 */
function bib_to_htmllist($bibfile, $split = SPLITONYEAR )  {
	global $bibtypes;
	global $bibsourcefile;

	$bibsourcefile = $bibfile;
	

	$cssfile = CSSFILE;
	$ret = '';

	// Parse the bibfile
	list($preamble, $strings, $entries, $undefinedStrings) =
		PHPBibHTML::parsebibfile($bibfile);

	// Sort the entries on TYPE and YEAR
	$reviewd = PHPBibHTML::sortentries($entries, $split);

	switch($split)  {
	case SPLITONTYPETHENYEAR:
		$ret = PHPBibHTML::genlist_ontype_thenyear($reviewd);
		break;
	case SPLITONYEARTHENTYPE:
		$ret = PHPBibHTML::genlist_onyear_thentype($reviewd);
		break;
	case SPLITONTYPE:
		$ret = PHPBibHTML::genlist_ontype($reviewd);
		break;
	case SPLITONYEAR:
		$ret = PHPBibHTML::genlist_onyear($reviewd);
		break;
	case DONTSPLIT:
	default:
		$ret = PHPBibHTML::genlist_nosplit($reviewd);
	}

	return $ret;
}

function attribution()  {
	return '<p class="attribution">Generated using <a ' .
		'href="http://www.open.ou.nl/hjo/software/php-bibHTML.htm">' .
		'php-bibHTML</a> by <a href="http://www.open.ou.nl/hjo/">' .
		'Hugo Jonker</a>, based on the <a ' .
		'href="http://bibliophile.sourceforge.net/">BiblioPhile</a> ' .
		'PHP BibTex parser.</p>' . "\n\n";
}

} // end of PHPBibHTML class

?>
